/*******************************************************/
/* file: ports.c                                       */
/* abstract:  This file contains the routines to       */
/*            output values on the JTAG ports, to read */
/*            the TDO bit, and to read a byte of data  */
/*            from the prom                            */
/*                                                     */
/* Notes: See the notes for micro.c for explanation of */
/*        the compiler switch  "DEBUG_MODE".           */
/*        This file has been customized for use in     */
/*        DEBUG_MODE with ONE of the following options:*/
/*        Define WIN95PP for Win95/98 parallel cable.  */
/*        Define WINDRVR for Win95/98/NT using WINDRVR */
/*        device driver for parallel cable.            */
/*        Define XSVFSIM for a simulation only .exe    */
/*******************************************************/
//////////////////////////////////////////////////////////////////////////////
// 2001/01/17
// Younjae modify WIN95PP mode to PCI9052....
//
#if defined(DEBUG_MODE) && ( defined(WIN95PP) || defined(WINDRVR) || defined(XSVFSIM) )
    #include    <stdio.h>
    #include    <stdlib.h>
    #include    <time.h>
    #if defined(WIN95PP)
        #include    <conio.h>
		#include <windows.h>
		#include "bgatel.h"
    #endif  /* WIN95PP */
    #if defined(WINDRVR)
        #include    "basjdrvif.h"
    #endif  /* WINDRVR */
#endif  /* DEBUG_MODE && (WIN95PP || WINDRVR) */

#include "ports.h"

#define CTRL_WORD 0x18780092

#ifdef DEBUG_MODE
extern FILE *in;
extern unsigned char *inbuf;
extern int xsvf_iDebugLevel;
#endif

#if defined(DEBUG_MODE) && ( defined(WIN95PP) || defined(WINDRVR) || defined(XSVFSIM) )
    static inPortType in_word;
    static outPortType out_word;
    static int once = 0;
	static int m_nBlueGateID = -1;
	static unsigned long *pReg = NULL;
    #if defined(WIN95PP)
        static unsigned short base_port = 0x378;
    #endif  /* WIN95PP */
    #if defined(WINDRVR)
        jdriver *JD;
        static int port = 0;
    #endif  /* WINDRVR */
    static long xsvf_lCountPerMsec;

    #ifdef  _MSC_VER
        #pragma optimize( "", off )
    #endif  /* _MSC_VER */
    void xsvfDelayLoop( long lCount )
    {
        /* WARNING:  Optimizing compilers may optimize away this empty loop! */
        while ( lCount-- ) ;
    }
    #ifdef  _MSC_VER
        #pragma optimize( "", on )
    #endif  /* _MSC_VER */

    void xsvfCalibrateDelay()
    {
        long    lCount;
        clock_t startClock;
        clock_t endClock;

        lCount      = 20000000L;
        startClock  = clock();
        xsvfDelayLoop( lCount );
        endClock    = clock();
        xsvf_lCountPerMsec=(lCount/(((endClock-startClock)*1000L)/CLOCKS_PER_SEC))+1;
    }

    void xsvfReportDelay()
    {
        int     i;
        clock_t startClock;
        clock_t endClock;

        printf( "    Timer Delay Count = %ld\n", xsvf_lCountPerMsec );
        for ( i = 20; i <= 100; i += 20 )
        {
            startClock  = clock();
            xsvfDelayLoop( i * xsvf_lCountPerMsec );
            endClock    = clock();
            printf( "    Timer:  Requested time = %3d msec;  "
                    "Real time = %3ld msec\n",
                    i, ( ( endClock - startClock ) * 1000L ) / CLOCKS_PER_SEC );
        }

    }
#endif  /* DEBUG_MODE && (WIN95PP || WINDRVR) */


/* if in debugging mode, then just set the variables */
void setPort(short p,short val)
{
#if defined(DEBUG_MODE) && ( defined(WIN95PP) || defined(WINDRVR) || defined(XSVFSIM) )
    //unsigned char   portStatusOn = 0;
    //unsigned char   portStatusOff= 0;
	static outPortType tmp_word;
	static unsigned int count = 0;

	if (once == 0) {
	//if(0){
        /* Initialize ports and calibrate timers (if necessary) */
		//out_word.bits.one = 1;
		//out_word.bits.zero = 0;
		once = 1;
        xsvfCalibrateDelay();
        if ( xsvf_iDebugLevel >= 4 )
        {
            xsvfReportDelay();
        }

        /* Check parallel cable connection */
        #if defined(WIN95PP)
            //_outp( (unsigned short) (base_port + 0), 0x40 );
            //portStatusOn    = (unsigned char) _inp( (unsigned short) (base_port + STATUS_OFFSET) );
            //_outp( (unsigned short) (base_port + 0), 0x00 );
            //portStatusOff   = (unsigned char) _inp( (unsigned short) (base_port + STATUS_OFFSET) );
			//tmp_word.value = ReadIO();
			//tmp_word.bits.io0dir=1; // out ( tck )
			//tmp_word.bits.io1dir=1; // out ( tms )
			//tmp_word.bits.io2dir=1; // out ( tdi )
			//tmp_word.bits.io3dir=0; // in  ( tdo )
			//tmp_word.bits.tck=0;
			//tmp_word.bits.tms=0;
			//tmp_word.bits.tdo=0;
			//WriteIO(tmp_word.value);
        #endif  /* WIN95PP */
    }

    /* Save the new port output values.  Write to port only on new TCK value */
	if (p==TMS)
		out_word.bits.tms = (unsigned char) val;
	if (p==TDI)
		out_word.bits.tdi = (unsigned char) val;
	if (p==TCK) {
		out_word.bits.tck = (unsigned char) val;
	#if defined(WIN95PP)
		out_word.value = out_word.value | CTRL_WORD;
		//printf(" %08lx : TCK = %d;  TMS = %d;  TDI = %d\n",
		//	out_word.value, ((int)(out_word.bits.tck)),
		//	((int)(out_word.bits.tms)),((int)(out_word.bits.tdi)) );
  		WriteIO(out_word.value);
		//_outp( (unsigned short) (base_port + 0), out_word.value );
		//
		if(count++==6000)
		{
			printf("#");
			count=0;
		}
    #endif  /* WIN95PP */
		}
#endif  /* DEBUG_MODE && (WIN95PP || WINDRVR) */
}


/* toggle tck LHL */
void pulseClock()
{
	setPort(TCK,0);  /* set the TCK port to low  - Set TDI & TMS */
                     /* port write has built-in delay */
	setPort(TCK,1);  /* set the TCK port to high - Latch in TDI & TMS */
                     /* port write has built-in delay */
	setPort(TCK,0);  /* set the TCK port to low  - Latch out TDO */
                     /* port write has built-in delay */
}


/* read in a byte of data from the prom */
void readByte(unsigned char *data)
{
	static i=0;
#ifdef DEBUG_MODE
	/* pretend reading using a file */
	*data   = (unsigned char)fgetc( in );
	//*data = (unsigned char)inbuf[i++];
#endif
}

/* read the TDO bit from port */
unsigned char readTDOBit()
{
#if defined(DEBUG_MODE) && ( defined(WIN95PP) || defined(WINDRVR) )
    #if defined(WIN95PP)
    	//in_word.value = (unsigned char) _inp( (unsigned short) (base_port + STATUS_OFFSET) );
		//
		in_word.value = ReadIO();
		//printf("%d",in_word.bits.tdo);
		//
    #endif  /* WIN95PP */
    return( in_word.bits.tdo );
#else
    return( 1 );
#endif  /* DEBUG_MODE && (WIN95PP || WINDRVR) */
}


/* Wait at least the specified number of microsec.                           */
/* Use a timer if possible; otherwise estimate the number of instructions    */
/* necessary to be run based on the microcontroller speed.  For this example */
/* we pulse the TCK port a number of times based on the processor speed.     */
void waitTime(long microsec)
{
#ifdef  DEBUG_MODE
    if ( microsec >= 1000L )
    {
        /* Use Windows Sleep().  Round up to the nearest millisec */
        _sleep( ( microsec + 999L ) / 1000L );
    }
    else
    {
        /* Use self-timed loop */
        xsvfDelayLoop( xsvf_lCountPerMsec * microsec / 1000000L );
    }
#endif  /* DEBUG_MODE */
}


int FindBlueGate()
{
	return BlueGateL_Find();
}

unsigned int OpenBlueGate(unsigned int id)
{
	outPortType ctrl_word;

	m_nBlueGateID=-1;

	if(BlueGateL_Open(id)==FALSE)
	{
		printf("No device : %d\n",id);
		return FALSE;
	}

	pReg = BlueGateL_GetRegisterPtr(id);

	/////////////////////////////////////////////////////////////
	// Initialization of PCI9052's User I/O
	//
	ctrl_word.value=ReadIO();
	
	ctrl_word.bits.io0dir=1; // out ( tck )
	ctrl_word.bits.io1dir=1; // out ( tms )
	ctrl_word.bits.io2dir=1; // out ( tdi )
	ctrl_word.bits.io3dir=0; // in  ( tdo )
	ctrl_word.bits.tck=0;
	ctrl_word.bits.tms=0;
	ctrl_word.bits.tdo=0;
	
	WriteIO(ctrl_word.value);

	printf("Initialize BlueGate-A [%d] ....OK.\n",id);

	m_nBlueGateID = id;
	once = 1;

	return TRUE;
}

void CloseBlueGate()
{
	outPortType ctrl_word;

	if(m_nBlueGateID==-1)
		return;

	/////////////////////////////////////////////////////////////
	// Initialization of PCI9052's User I/O
	//
	ctrl_word.value=ReadIO();
	
	ctrl_word.bits.io0dir=0; // in ( tck )
	ctrl_word.bits.io1dir=0; // in ( tms )
	ctrl_word.bits.io2dir=0; // in ( tdi )
	ctrl_word.bits.io3dir=0; // in ( tdo )
	ctrl_word.bits.tck=0;
	ctrl_word.bits.tms=0;
	ctrl_word.bits.tdo=0;
	
	WriteIO(ctrl_word.value);

	printf("Close BlueGate-L [%d] ....OK.\n",m_nBlueGateID);

	BlueGateL_Close(m_nBlueGateID);

	m_nBlueGateID = -1;
	once = 0;
}

unsigned int ReadIO(void)
{
	if(m_nBlueGateID==-1)
		return 0;

	//return BlueGate_ReadRegister(m_nBlueGateID,0x50);
	return pReg[0x50/4];
}

void WriteIO(unsigned int data)
{
	if(m_nBlueGateID==-1)
		return;

	//BlueGate_WriteRegister(m_nBlueGateID,0x50,data);
	pReg[0x50/4] = data;
}
